/* wrs_umon_client.c : white rabbit switch micro monitor - client
*
*  author: Dietrich Beck, GSI-Darmstadt
*/
#define VERSIONSTR  "0.01b"
/*
*  history: This program was developed as a exercise. The main task is to provide
*  very basic monitoring of a White Rabbit switch. The communication layer is DIM.
*  
*  The code retrieving information from the servo and the ports has been taken from
*  wr_mon, a command line program included in the White Rabbit switch software 3.1.
*  30-JAN-2012 first version
*
*  purpose: subscribe and display WR switch status
*
*  License Agreement for this software:
*
*  Copyright (C)
*  GSI Helmholtzzentrum für Schwerionenforschung GmbH
*  Planckstr. 1
*  64291 Darmstadt
*  Germany
*
*  Contact: D.Beck@gsi.de 
*  
*  This program is free software; you can redistribute it and/or modify it under the terms of the 
*  GNU General Public License as published by the Free Software Foundation; either version 2 of 
*  the license, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
*  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
*  the GNU General License for more details (http://www.gnu.org).
*  
*  GSI Helmholtzzentrum für Schwerionenforschung GmbH
*  Planckstr. 1, 64291 Darmstadt, Germany
*  For all questions and ideas contact: D.Beck@gsi.de
*  Last update: 31-JAN-2012
*/

/* standard includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* DIM includes */
#include <dic.h>

/* umon constants */
#define SERVLEN_S 9  //add an extra character to be on the safe side
#define SERVLEN_L 33
#define MAXLEN 133

/* DIM service IDs */
int versionID = -1;
int syncStateID = -1;
int portStateID = -1; 

/* DIM services */ 
char version[SERVLEN_S];
char syncState[SERVLEN_S];
char portState[SERVLEN_L];

/* global stuff */
int no_link = -1;
char myName[SERVLEN_L];

/*----------------------------------------------------------------
  message : print a message to stdout
  ----------------------------------------------------------------*/
void message(char *msg)
{
  char buff[MAXLEN];

  sprintf(buff, "%s - umon: %s\n", myName, msg);
  printf(buff);
  fflush(stdout); /* required in case of redirected output  */
} /* message */


/*----------------------------------------------------------------
  call back for services containing strings
 -----------------------------------------------------------------*/
void stringCB(tag, bufferp, size)
long *tag;
char *bufferp;
 int *size;
{
  char buff[MAXLEN];
  char anotherBuff[MAXLEN];

  sprintf(anotherBuff, "Ohps!");

  if(bufferp[0] == -1) /* no connection to server */
  {
	if (*tag == 1) /* only print NOLINK for one service */ 
	{
	  sprintf(buff, "NOLINK");
	  message(buff);
	}
  }
  else /* connection to server */
  {
	if (*tag == 1) sprintf(buff, "established link to wrs_umon_server version %s", bufferp);
	if (*tag == 2) sprintf(buff, "sync state: %s", bufferp);
	if (*tag == 3) sprintf(buff, "port state: %s", bufferp);
	message(buff);
  }


} /* stringCB */ 

/*----------------------------------------------------------------
  subscribe_dim_services
  ----------------------------------------------------------------*/
void subscribe_dim_services(char *serverID)
{
  char   buff[MAXLEN];

  //version number
  sprintf(buff, "%s/wrs_umon/version", serverID);
  versionID = dic_info_service(buff, MONITORED, 0, version, SERVLEN_S, stringCB, 1, &no_link, sizeof(no_link));

  //sync state
  sprintf(buff, "%s/wrs_umon/syncState", serverID);
  syncStateID = dic_info_service(buff, MONITORED, 0, version, SERVLEN_S, stringCB, 2, &no_link, sizeof(no_link));

  //port state
  sprintf(buff, "%s/wrs_umon/portState", serverID);
  portStateID = dic_info_service(buff, MONITORED, 0, portState, SERVLEN_L, stringCB, 3, &no_link, sizeof(no_link));
  //wrsmon ID

} /* subscribe_dim_services */


int main(int argc, char **argv)
{
  char   buff[MAXLEN];

  if (argc < 2) 
	{
	  printf("Usage: wrs_umon_clientr IDENTIFIER\n");
	  printf("IDENTIFIER must be a unique identifier like IP number or hostname.\n");
	  exit(1);
	}
  sprintf(myName, "%s", argv[1]);
  sprintf(buff, "white rabbit switch micro monitor: client version %s", VERSIONSTR);
  message(buff);

  /* start doing things */
  subscribe_dim_services(myName);

  /* what forever until CTRL+C is pressed */
  for(;;) sleep(10000000);

  return(0);
} /* main */
